/*
Class DefDialog

Copyright (C) 2003 Alwen Tiu

   This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

   This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

   You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software Foundation,
Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

import javax.swing.JSplitPane;
import javax.swing.JTable;
import javax.swing.JPanel;
import javax.swing.JTextPane;
import javax.swing.JButton;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.JScrollPane;
import javax.swing.JDialog;
import java.awt.*;
import java.awt.event.*;

class DefDialog extends JDialog
                   implements ActionListener    {
    private JTable table;	                   
    private String clause = null;
    private boolean selectAll = false;
    private boolean select = false;
    private boolean inst = false;
	private String[] clauseList;
	private JTextPane selText;
	

     
    public String getClause() {
        return clause;
    }
	
	public boolean isInst() { return inst; }
	public boolean isSelect() { return select; }
	public boolean isSelectAll() { return selectAll;}
	
	
    public DefDialog(Frame aFrame, String[] cList) {
        super(aFrame, true);

        String[] col = {"Clause"};
        
        if(cList == null) return;
        
        if(cList.length == 0) return;
        
        clauseList = cList;
        
        
        setTitle("Choose a definition clause");
		String[][] data = new String[cList.length][1];
		
		for(int i=0; i < cList.length; ++i)
		{
			data[i][0] = cList[i];	
		}
		
		table = new JTable(data, col);
        table.setPreferredScrollableViewportSize(new Dimension(400, 70));

        table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

        ListSelectionModel rowSM = table.getSelectionModel();
        rowSM.setSelectionInterval(0,0);
        rowSM.addListSelectionListener(new ListSelectionListener() {
		    public void valueChanged(ListSelectionEvent e) {
        		//Ignore extra messages.
        		if (e.getValueIsAdjusting()) return;
        
        		ListSelectionModel lsm =
            			(ListSelectionModel)e.getSource();
        		if (lsm.isSelectionEmpty()) {
        			return;    
        		} else {
            		int selectedRow = lsm.getMinSelectionIndex();
            		selText.setText(clauseList[selectedRow]);
            		selText.setCaretPosition(0);
        		}
    		}
		});

        JScrollPane listPane = new JScrollPane(table);
        //listPane.setMinimumSize(new Dimension(400, 100));
        
        JPanel btnPanel = new JPanel();
        
        JButton btnSel = new JButton("Select");
        JButton btnInst = new JButton("Instantiate");
        JButton btnAll  = new JButton("Try All");
        JButton btnCancel = new JButton("Cancel");
		        
        selText = new JTextPane();
        selText.setText(clauseList[0]);
        selText.setCaretPosition(0);
        selText.setMargin(new Insets(5,5,5,5));
		
		
        JScrollPane curSelView = new JScrollPane(selText);
        curSelView.setMinimumSize(new Dimension(400, 70));
        
        
        btnSel.setActionCommand("Select");
        btnSel.addActionListener(this);
        btnInst.setActionCommand("Instantiate");
		btnInst.addActionListener(this);
		btnAll.setActionCommand("Try All");
		btnAll.addActionListener(this);
		btnCancel.setActionCommand("Cancel");
		btnCancel.addActionListener(this);
		
		btnPanel.add(btnSel);
		btnPanel.add(btnInst);
		btnPanel.add(btnAll);
		btnPanel.add(btnCancel);
		
		JPanel textAndBtn = new JPanel();
		
		textAndBtn.setLayout(new BorderLayout());
		textAndBtn.add(curSelView, BorderLayout.CENTER);
		textAndBtn.add(btnPanel, BorderLayout.SOUTH);
		
        JPanel mainPanel = new JPanel();
        
        JSplitPane splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
        splitPane.setTopComponent(listPane);
        splitPane.setBottomComponent(textAndBtn);
        splitPane.setMinimumSize(new Dimension(500, 300));
        splitPane.setPreferredSize(new Dimension(500,300));
        splitPane.setDividerLocation(150);
        
//        mainPanel.setLayout(new BorderLayout());
//        mainPanel.add(listPane, BorderLayout.CENTER);
//        mainPanel.add(textAndBtn, BorderLayout.SOUTH);
//        mainPanel.setMinimumSize(new Dimension(500, 200));
//        mainPanel.setPreferredSize(new Dimension(500, 150));
        
        //Make this dialog display it.
//        setContentPane(mainPanel);
		setContentPane(splitPane);
		table.doLayout();
    }

    
    public void actionPerformed(ActionEvent e) {
	    
        if("Instantiate".equals(e.getActionCommand()))
        {
	         int i = table.getSelectedRow();
	         if(i >= 0)
	         	clause = clauseList[i];
			  else clause = null;	
			  inst = true; 
			  clearAndHide();    	
	         	
        }
        else if("Select".equals(e.getActionCommand()))
        {
	         int i = table.getSelectedRow();
	         if(i >= 0)
	         	clause = clauseList[i];
			  else clause = null;	
			  
			  select = true;			   
			  clearAndHide();    		        
        }
        else if("Try All".equals(e.getActionCommand()))
        {
	        clause = null;
	        
	        selectAll = true;
	        clearAndHide();
        }
        else if("Cancel".equals(e.getActionCommand()))
        {
	    	clearAndHide();    
        }
    }

    public void clearAndHide() {
        
        setVisible(false);
    }
}

