/*
Class DerivationView

Copyright (C) 2003 Alwen Tiu

   This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

   This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

   You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software Foundation,
Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

import javax.swing.JTree;
import javax.swing.*;
import javax.swing.tree.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.FileDialog;
import java.io.*;
import java.lang.*;
import javax.swing.event.*;
import javax.swing.text.*;
import java.net.URL;

public class DerivationView extends JPanel
{
	private JTree tree;
	private DefaultMutableTreeNode cur_node;	
	private Linc parent;
    private DefaultTreeModel treeModel;
	
	
	public DerivationView(Linc p)
	{
		cur_node = null;
		
		parent = p;
       	Sequent topSeq = new Sequent("seq Root. \ngoal true.");	

        //Create the nodes.
        DefaultMutableTreeNode top = new DefaultMutableTreeNode(topSeq);
        
		treeModel = new DefaultTreeModel(top);
		cur_node = top;
		
        //Create a tree that allows one selection at a time.
		tree = new JTree(treeModel);		
        
        tree.getSelectionModel().setSelectionMode
                (TreeSelectionModel.SINGLE_TREE_SELECTION);

        tree.setCellRenderer(new MyTreeRenderer());
        
        //Listen for when the selection changes.
        tree.addTreeSelectionListener(new TreeSelectionListener() {
            public void valueChanged(TreeSelectionEvent e) {
                DefaultMutableTreeNode node = (DefaultMutableTreeNode)
                                   tree.getLastSelectedPathComponent();
				
								
                if (node == null) return;

                if(cur_node != null)
                {
               		Sequent seq = (Sequent) cur_node.getUserObject();
               		seq.parse(parent.getSequent());
               		//seq.setMainForm(parent.getMainFo());
               		//seq.setExp(parent.getExp());
               		
               		cur_node.setUserObject(seq);
               		// treeModel.reload(cur_node);	                
            	}
              
                Object nodeInfo = node.getUserObject();
                Sequent seqNode = (Sequent) nodeInfo;
				cur_node = node;          
				parent.displaySequent(seqNode);      
                //seqView.displaySequent(seqNode);
                //tabPane.setSelectedIndex(0);
            }
        });

        //Create the scroll pane and add the tree to it. 
        JScrollPane treeView = new JScrollPane(tree);	
        setLayout(new BorderLayout());
        add(treeView, BorderLayout.CENTER);
        
        setBorder(BorderFactory.createCompoundBorder(
			BorderFactory.createTitledBorder("Derivation"), 
			BorderFactory.createEmptyBorder(5,5,5,5)));
	}
	
	public void newDerivation()
	{
		cur_node = null;
				
		Sequent topSeq = new Sequent("seq Root. \ngoal true.");
        //Create the nodes.
        DefaultMutableTreeNode top = new DefaultMutableTreeNode(topSeq);
        
		treeModel = new DefaultTreeModel(top);
		
		tree.setModel(treeModel);		
		cur_node = (DefaultMutableTreeNode ) treeModel.getRoot();	
		parent.displaySequent(topSeq);
		tree.updateUI();		
	}
	
	public boolean writeFile(File fpath)
	{
		if(cur_node == null)
			return false;
			
       	Sequent seq = (Sequent) cur_node.getUserObject();
		seq.parse(parent.getSequent());
        cur_node.setUserObject(seq);
		            		
		try {
			ObjectOutputStream outf = new ObjectOutputStream(new FileOutputStream(fpath));
			outf.writeObject(treeModel);
			outf.close();
			return true;
		}
		catch(IOException e)
		{
			parent.errMsg("Writing file " + fpath.getName() + " failed");
			return false;				
		}	
	}
	
	public boolean loadFile(File fpath)
	{
		DefaultTreeModel model = null;

		try{
			ObjectInputStream inf = new ObjectInputStream(new FileInputStream(fpath));
			model = (DefaultTreeModel) inf.readObject();
			inf.close();
			treeModel.reload();
		}
		catch(IOException e)
		{
			parent.errMsg("IO exception in loading file " + fpath.getName());
			
			return false;
		}	
		catch(ClassNotFoundException k)
		{
			parent.errMsg("Wrong file format");
			return false;
		}		
		if (model == null)
		{
			parent.errMsg("Loading file " + fpath.getName() + " failed");
			return false;
		}		
		treeModel = model;
		tree.setModel(treeModel);
		tree.updateUI();
		cur_node = null;		
		return true;
	}
	
	public void addNewSequents(String rule, String mainFo, String exp, String[] ls)
	{
		System.out.println("entering addNewSequents");
		System.out.println("Number of sequents: ");
		System.out.println(ls.length);
		
	    
	    if(cur_node == null) 
	    {
		 	System.err.println("addNewSequents: cur_node is null");
		 	return;		 	
	    }
	    
	    TreePath path = new TreePath(treeModel.getPathToRoot(cur_node));
        if( path == null ){
	        System.out.println("addNewSequents: path is null");        
	        return;
        }

        DefaultMutableTreeNode node = cur_node;
        
        DefaultMutableTreeNode cnode = null;
        int n = treeModel.getChildCount(node);
        
        while(n > 0) { 
			cnode = (DefaultMutableTreeNode) treeModel.getChild(node, 0);
	     	treeModel.removeNodeFromParent(cnode);   
	     	n--;
        }
        
        int countSeq = 0;
                
		for(int i=0; i < ls.length; ++i)
		{
			if(ls[i].indexOf("seq") != -1)
			{
				++countSeq;
            	DefaultMutableTreeNode newNode = 
            		new DefaultMutableTreeNode(new Sequent(ls[i]));
            	treeModel.insertNodeInto(newNode,node,node.getChildCount());   
				TreePath newPath = path.pathByAddingChild(newNode);
            	tree.makeVisible(newPath);
			}            	
        }
        
        Sequent seq = (Sequent) node.getUserObject();
        if(countSeq == 0) 
        {
	        seq.setLeaf(Boolean.TRUE);
        }
        else 
        {
	        seq.setLeaf(Boolean.FALSE);
        }
        
        seq.setMainForm(mainFo);
        seq.setRule(rule);
        seq.setExp(exp);
        node.setUserObject(seq);	                                         
        
        if(countSeq == 0)
        {
        	tree.setSelectionPath(path);
        	tree.scrollPathToVisible(path);
        }
		else
		{
			//cur_node = (DefaultMutableTreeNode) treeModel.getChild(cur_node,0);
	        TreePath newPath = path.pathByAddingChild(treeModel.getChild(cur_node,0));
	        tree.setSelectionPath(newPath);	   
	        tree.scrollPathToVisible(newPath);     				        
		}        	
		tree.updateUI();
	}
	
	public 	DefaultMutableTreeNode getCurrentNode()
	{
		return cur_node;	
	}
    private class MyTreeRenderer extends DefaultTreeCellRenderer {
        ImageIcon leafIcon;
		ImageIcon nodeIcon;
		ImageIcon openNodeIcon;
		
        public MyTreeRenderer() {
			URL url = Linc.class.getResource("leaf.gif"); 			
			Image img = Toolkit.getDefaultToolkit().getImage(url); 
	        
			leafIcon = new ImageIcon(img);
			
			url = Linc.class.getResource("node.gif"); 			
			img = Toolkit.getDefaultToolkit().getImage(url); 			
			nodeIcon = new ImageIcon(img);
			
			url = Linc.class.getResource("opennode.gif"); 			
			img = Toolkit.getDefaultToolkit().getImage(url); 			
			openNodeIcon = new ImageIcon(img);
			
            //leafIcon = new ImageIcon("leaf.gif");
            //nodeIcon = new ImageIcon("node.gif");
            //openNodeIcon = new ImageIcon("opennode.gif");
        }

        public Component getTreeCellRendererComponent(
                            JTree tree,
                            Object value,
                            boolean sel,
                            boolean expanded,
                            boolean leaf,
                            int row,
                            boolean hasFocus) {

            super.getTreeCellRendererComponent(
                            tree, value, sel,
                            expanded, leaf, row,
                            hasFocus);
            if (isSequentLeaf(value)) {
                setIcon(leafIcon);                
            } 
			else if (!leaf)
			{
				setIcon(nodeIcon);	
			}
			else setIcon(openNodeIcon);
            return this;
        }

        protected boolean isSequentLeaf(Object value) {
            DefaultMutableTreeNode node =
                    (DefaultMutableTreeNode)value;
            Sequent nodeSeq = 
                    (Sequent)(node.getUserObject());
            Boolean isleaf = nodeSeq.getLeaf();
            if (isleaf.booleanValue()) {
                return true;
            } 

            return false;
        }
    }

		
}