/*
Class SequentView

Copyright (C) 2003 Alwen Tiu

   This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

   This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

   You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software Foundation,
Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

import javax.swing.JTree;
import javax.swing.*;
import javax.swing.tree.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.FileDialog;
import java.io.*;
import java.lang.*;
import javax.swing.event.*;
import javax.swing.text.*;

public class SequentView extends JPanel {
	private LincDoc textDoc, mainFoDoc, expDoc;
	private JTextPane textPane;
	private JTextPane textMainFo;
	private JTextPane textExp;
	
	public SequentView()
	{
		textDoc = new LincDoc();
		
        textPane = new JTextPane(textDoc);
        textPane.setEditable(true);
		
        // prevent auto-wrapping of JTextPane
        JPanel noWrap = new JPanel();
        noWrap.setLayout(new BorderLayout());
        noWrap.add(textPane);
        JScrollPane textScrollView = new JScrollPane(noWrap);
        textScrollView.setMinimumSize(new Dimension(400,300));
        
        
        // add main formula and user input expression

        JLabel lbMainFo = new JLabel("Main formula");        
        mainFoDoc = new LincDoc();
        textMainFo = new JTextPane(mainFoDoc);
        textMainFo.setEditable(false);
        noWrap = new JPanel();
        noWrap.setLayout(new BorderLayout());
        noWrap.add(textMainFo);
        
        JScrollPane mainFoScroll = new JScrollPane(noWrap);
        mainFoScroll.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED );
        JPanel mainFoView = new JPanel();
        mainFoView.setLayout(new BorderLayout());
        mainFoView.add(mainFoScroll, BorderLayout.CENTER);
        mainFoView.add(lbMainFo, BorderLayout.NORTH);
        mainFoView.setBorder(BorderFactory.createCompoundBorder(
			BorderFactory.createTitledBorder(""), 
			BorderFactory.createEmptyBorder(5,5,5,5)));
        
        JLabel lbExp = new JLabel("User input expression");
        expDoc = new LincDoc();
        textExp = new JTextPane(expDoc);              
		textExp.setEditable(false);
		
        noWrap = new JPanel();
        noWrap.setLayout(new BorderLayout());
        noWrap.add(textExp);

        JScrollPane expScroll = new JScrollPane(noWrap);
        expScroll.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED );
		JPanel expView = new JPanel();
		expView.setLayout(new BorderLayout());
		expView.add(expScroll, BorderLayout.CENTER);
		expView.add(lbExp, BorderLayout.NORTH);  
        expView.setBorder(BorderFactory.createCompoundBorder(
			BorderFactory.createTitledBorder(""), 
			BorderFactory.createEmptyBorder(5,5,5,5)));
		      
        
        JSplitPane infoView = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);        
        infoView.setLeftComponent(mainFoView);
        infoView.setRightComponent(expView);
        infoView.setDividerLocation(300);
		infoView.setOneTouchExpandable(true);
        //infoView.setBorder(BorderFactory.createCompoundBorder(
		//	BorderFactory.createTitledBorder(""), 
		//	BorderFactory.createEmptyBorder(5,5,5,5)));
		        
        JSplitPane splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
        splitPane.setTopComponent(textScrollView);
        splitPane.setBottomComponent(infoView);
        splitPane.setDividerLocation(300);
        splitPane.setOneTouchExpandable(true); 
                
        setLayout(new BorderLayout());
        add(splitPane, BorderLayout.CENTER);
        
	}
	
	public void displaySequent(Sequent s)
	{
		textPane.setText(s.getSeq());		
		textPane.setCaretPosition(0);

		textMainFo.setText(s.getMainForm());		
		textMainFo.setCaretPosition(0);

		textExp.setText(s.getExp());
		textExp.setCaretPosition(0);
				
	}	
	
	public String getSequent() { return textPane.getText(); }
	public String getMainFo() { return textMainFo.getText();}
	public String getExp() { return textExp.getText();}
	
	public void setSequent(String s) 
	{ 
		textPane.setText(s); 
		textPane.setCaretPosition(0);
	}
	
	
	public void writeFile(FileWriter outf)
	{
		try {
			textPane.write(outf);	
		} 
		catch(IOException e)
		{
			System.out.println("SequentView::writeFile IOException");	
		}
	}
	
}	