% Module linc
%
% Copyright (C) 2003 Alwen Tiu
%
%   This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or (at
% your option) any later version.
% 
%   This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
%   You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software Foundation,
% Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


% Module: linc
% Author: Alwen Tiu
% Date: 8/26/03
% Description: this provides command line interface to the module infer.

module linc.

accum_sig infer.
accumulate list, control, stt, printer, infer.

         
constant X :- 
   current_def (def Nm Kn Sig Cls), 
   member X Sig.

% commands

type execute_cmd cmd -> int -> list sequent -> o.
type prove_seq int -> list sequent -> o.

start_linc File :-
   load_def File Def, !,
   (current_def Def => prove_seq 0 nil).


prove_seq N Seqs :-
   current_def (def Nm _ _ _),
   term_to_string N Nstr,
   print "\n[Definition: ", print Nm, 
   print ", ", 
   print Nstr, print " sequent(s)]\n> ",
   input_line std_in S, 
   Str is S ^ ".",
   if (string_to_term Str Cmd) 
      (execute_cmd Cmd N Seqs)
      (print "Invalid command. Type 'help' to see available commands.\n",
       prove_seq N Seqs).

execute_cmd (load File) N LSeq :-
   !, if (load_seq File LSeq1 M) 
      (print "Sequent loaded!\n", prove_seq M LSeq1) 
      (print "Loading sequent failed!\n", prove_seq N LSeq).
execute_cmd (save File) N LSeq :-
   !, 
   if (N = 0)
      (print "No sequents to save!\n")
      (
        open_out File Out,
        print_ListSequent Out LSeq,
        print "Sequents saved.\n",
        close_out Out
       ),
   prove_seq N LSeq.
      
execute_cmd (display all) N LSeq :-
   !, 
   print "\n", print_NumberedSequent 1 std_out LSeq,
   prove_seq N LSeq.

execute_cmd (display I) N LSeq :-
   !, 
   if (get_nth_seq I LSeq Seq RSeq)
      (   
        print "\n", print_sequent std_out Seq,
        print "\n"
       )
       (print "\nSequent not found.\n"),
   prove_seq N LSeq.

execute_cmd showdef N LSeq :-
   !, print "\n", current_def Def,
   print_def std_out Def,
   print "\n", prove_seq N LSeq.
            
execute_cmd quit _ _ :- !.
execute_cmd help N L :- !, 
   print "Commands:\n",
   print "load \"file name\"    load a file containing sequents\n",
   print "save \"file name\"    save current sequents to a file\n",
   print "prove n             prove the n-th sequent\n",
   print "display n           display the n-th sequent\n",
   print "display all         display all sequents\n",
   print "quit                quit this interpreter\n",
   print "help                display help messages\n",
   prove_seq N L.

execute_cmd (prove I) N LSeq :- 
   !, if (I > N ; I =< 0)
         (
           print "Sequent number out of range\n",
           prove_seq N LSeq)
         (
           get_nth_seq I LSeq Seq RSeq,
           print_sequent std_out Seq,
           if (prove_nth_seq Seq LSeq' NP) 
              (
                 print "Rule application succeeded. Following premises are added:\n",
                 print_ListSequent std_out LSeq',
                 N' is (N + NP) - 1,
                 append RSeq LSeq' NewLSeq,
                 prove_seq N' NewLSeq
 			  )
 			  (
 			     print "Rule application failed.\n",
 			     prove_seq N LSeq
 			  )            
         ).
         
execute_cmd S N L :- 
   print "Invalid command. Type 'help' to see available commands.\n", 
   prove_seq N L.
   

type get_nth_seq int -> list sequent -> sequent -> list sequent -> o.

get_nth_seq 1 [Seq | R] Seq R :- !.
get_nth_seq N [S | L] Seq [S|R] :-
   N > 1,
   N' is N - 1,
   get_nth_seq N' L Seq R.

type prove_nth_seq sequent -> list sequent -> int -> o.

prove_nth_seq Seq LSeq N :-
   print "\nEnter rule to apply (or 'help' to see available rules): ",
   input_line std_in Rule,
   infer Rule Seq LSeq N.
      
   
type infer string -> sequent -> list sequent -> int -> o.

% Description:
%  Inference rules for FOLDN.
%  Take a sequent as input and produces a list of sequents 
%  as output.

infer "help\n" Seq LSeq N :-
   print "\nInference rules: \n",
   print "init, top, bot, eq-r, and-r, or-r1, or-r2, imp-r, def-r, coind-r\n",
   print "contr-l, weak-l, eq-l, and-l, or-l, imp-l, ind-l, def-l, focus\n",
   prove_nth_seq Seq LSeq N.
   
infer "init\n" (seq Nm LFv Hyp G)  nil 0 :-
   member G Hyp.
   
infer "top\n" (seq Nm LFv Hyp ltrue) nil 0.

infer "bot\n" (seq Nm LFv Hyp G) nil 0 :-
   member lfalse Hyp.

infer "eq-r\n" (seq Nm LFv Hyp (leq @ T @ T)) nil 0.
         
infer "and-r\n" (seq Nm LFv Hyp G) LSeq N :-
   inf inf_andr G (seq Nm LFv Hyp G) LSeq N.
   
infer "or-r1\n" (seq Nm LFv Hyp G) LSeq N :-
   inf inf_orr1 G (seq Nm LFv Hyp G) LSeq N.

infer "or-r2\n" (seq Nm LFv Hyp G) LSeq N :-
   inf inf_orr2 G (seq Nm LFv Hyp G) LSeq N.
         
infer "imp-r\n" (seq Nm LFv Hyp G) LSeq N :-
   inf inf_impr G (seq Nm LFv Hyp G) LSeq N.

infer "sigma-r\n" (seq Nm LFv Hyp G) LSeq N :-
   inf inf_sigmar G (seq Nm LFv Hyp G) LSeq N.
   
infer "pi-r\n" (seq Nm LFv Hyp G) LSeq N :-
   inf inf_pir G (seq Nm LFv Hyp G) LSeq N.
   
infer "def-r\n" (seq Nm LFv Hyp G) LSeq N :-
   inf inf_defr G (seq Nm LFv Hyp G) LSeq N.

infer "coind-r\n" (seq Nm LFv Hyp G) LSeq N :-
   inf inf_coindr G (seq Nm LFv Hyp G) LSeq N.
      
infer "contr-l\n" (seq Nm LFv Hyp G) LSeq N :-
   choose_hyp Hyp H Hyp',
   inf inf_contrl H (seq Nm LFv Hyp G) LSeq N.

infer "weak-l\n" (seq Nm LFv Hyp G) LSeq N :-
   choose_hyp Hyp H Hyp',
   inf inf_weakl H (seq Nm LFv Hyp' G) LSeq N.

            
infer "and-l\n" (seq Nm LFv Hyp G) LSeq N :-
   choose_hyp Hyp H Hyp',
   inf inf_andl H (seq Nm LFv Hyp' G) LSeq N.

infer "or-l\n" (seq Nm LFv Hyp G) LSeq N :-
   choose_hyp Hyp H Hyp',
   inf inf_orl H (seq Nm LFv Hyp' G) LSeq N.

infer "imp-l\n" (seq Nm LFv Hyp G) LSeq N :-
   choose_hyp Hyp H Hyp',
   inf inf_impl H (seq Nm LFv Hyp' G) LSeq N.

infer "sigma-l\n" (seq Nm LFv Hyp G) LSeq N :-
   choose_hyp Hyp H Hyp',
   inf inf_sigmal H (seq Nm LFv Hyp' G) LSeq N.

infer "pi-l\n" (seq Nm LFv Hyp G) LSeq N :-
   choose_hyp Hyp H Hyp',
   inf inf_pil H (seq Nm LFv Hyp' G) LSeq N.
            
infer "eq-l\n" (seq Nm LFv Hyp G) LSeq N :-
   choose_hyp Hyp H Hyp',
   inf inf_eql H (seq Nm LFv Hyp' G) LSeq N.

infer "def-l\n" (seq Nm LFv Hyp G) LSeq N :-
   choose_hyp Hyp H Hyp',
   inf inf_defl H (seq Nm LFv Hyp' G) LSeq N.

infer "ind-l\n" (seq Nm LFv Hyp G) LSeq N :- 
   choose_hyp Hyp H Hyp',
   inf inf_indl H (seq Nm LFv Hyp' G) LSeq N.

infer "focus\n" Seq LSeq N :-
   print "\nUsing focusing tactic ...\n",
   focus Seq LSeq N.
        
% ---

type choose_hyp list tm -> tm -> list tm -> o.
type print_hyp int -> list tm -> o.

choose_hyp Hyp T Hyp' :- 
   print "\nAvailable hypotheses:\n",
   print_hyp 1 Hyp, 
   print "\nChoose a formula from the list: ",
   input_line std_in Str,
   S is Str ^ ".",
   string_to_term S N,
   print "\n",
   pick_hyp N Hyp T Hyp'.

print_hyp N nil.
print_hyp N [T|L] :-
   term_to_string N Str,
   print "[", print Str, print "] ",
   print_lterm std_out 1 T,
   print "\n",
   N' is N + 1,
   print_hyp N' L.
   
type pick_hyp int -> list tm -> tm -> list tm -> o.

pick_hyp 1 [T|Hyp] T Hyp :- !.
pick_hyp N [T|Hyp] R [T|Hyp'] :-
   N' is N - 1,
   pick_hyp N' Hyp R Hyp'.

% -----
