% Module lincgrammar
%
% Copyright (C) 2003 Alwen Tiu
%
%   This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or (at
% your option) any later version.
% 
%   This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
%   You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software Foundation,
% Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


% Module: lincgrammar
% Author: Alwen Tiu
% Date: 8/26/03
% Description: grammar specification of lambda terms, sequents and definitions.
%    Making use of lambdayacc module.

module lincgrammar.

accum_sig lambdayacc.
accumulate lambdayacc.
accumulate stt.

terminal_list 
[
   linc_symAbs, lparen, rparen, (id S), linc_symTrue,
   linc_symFalse, linc_symAnd, linc_symOr, linc_symImp,
   linc_symAll, linc_symExists, linc_symDef,
   linc_symKind, linc_symType, linc_symArr, 
   linc_colon, linc_comma, linc_dot, linc_symDefName, 
   linc_symSeq, linc_symHyp, linc_symGoal, linc_symEq, linc_symFormt,
   linc_symInd, linc_symCoInd
].

nonterm_list [(ary1 linc_defClause), (ary1 linc_term), 
   (ary1 linc_type), (ary1 linc_typeDcl), (ary1 linc_typeDcls), (ary1 id_list),
   (ary1 linc_kindDcls), (ary1 linc_kindDcl), 
   (ary1 linc_defHead), (ary1 linc_defBody), (ary1 linc_defn), (ary1 linc_defnb),
   (ary1 linc_sequent), (ary1 linc_seqBody), (ary1 linc_seqBody2), (ary1 linc_tmList),
   (ary1 linc_root), (ary0 telparen), (ary1 nontermst),
   (ary1 linc_seqList)].

printname _ "seq" linc_symSeq.
printname _ "hyp" linc_symHyp.
printname _ "goal" linc_symGoal.
printname _ "(" lparen.
printname _ ")" rparen.
printname _ "\\" linc_symAbs.
printname _ "true" linc_symTrue.
printname _ "false" linc_symFalse.
printname _ "&" linc_symAnd.
printname _ ";" linc_symOr.
printname _ "=>" linc_symImp.
printname _ "pi" linc_symAll.
printname _ "sigma" linc_symExists.
printname _ "<=>" linc_symDef.
printname _ ":" linc_colon.
printname _ "," linc_comma.
printname _ "." linc_dot.
printname _ "kind" linc_symKind.
printname _ "def" linc_symDefName.
printname _ "type" linc_symType.
printname _ "->" linc_symArr.
printname _ "=" linc_symEq.
printname _ "o" linc_symFormt.
printname _ "ind" linc_symInd.
printname _ "coind" linc_symCoInd.

comment_list ["%"].
tokchar S :- alltyp S, C is string_to_int S,
	     not (C = 46; C = 44; C = 41; C = 40; C = 59; C = 91; C = 93; C = 124; C = 58; C = 92; C = 34; C = 126).


start_symbol (linc_root R).	     
%start_symbol (linc_sequent Seq).	     
%start_symbol (linc_defn L).
%start_symbol(linc_defClause D).
%start_symbol (linc_term T).
%start_symbol (linc_type T).
%start_symbol (linc_typeDcls D).
%start_symbol (linc_kindDcls D).

cfg    
[
   rule ((linc_root LincRoot1) ==> [linc_seqList LincRoot2]) (LincRoot1 = LincRoot2),
   rule ((linc_root LincRoot3) ==> [linc_defn LincRoot4]) (LincRoot3 = LincRoot4),

   rule ((linc_seqList SeqList) ==> 
       [linc_sequent SSeq, linc_seqList SeqList'])
       (SeqList = [SSeq | SeqList']),
   rule ((linc_seqList SeqList1) ==>
       [linc_sequent SSeq1]) (SeqList1 = [SSeq1]),
   rule ((linc_sequent SeqRoot) ==> 
          [linc_symSeq, id SeqName, linc_dot, linc_seqBody LSeqHyp, 
           linc_seqBody2 SeqGoal])
        (SeqRoot = (linc_seqRoot SeqName LSeqHyp SeqGoal)),
   rule ((linc_sequent SeqRoot1) ==>
          [linc_symSeq, id SeqName1, linc_dot, linc_seqBody2 SeqGoal1])
        (SeqRoot1 = linc_seqRoot SeqName1 nil SeqGoal1),

   rule ((linc_seqBody2 Goal2) ==>
         [linc_symGoal, linc_term TGoal, linc_dot]) (Goal2 = TGoal),
        
   rule ((linc_seqBody LHyps) ==>
          [(linc_typeDcl LTyHyp), linc_seqBody LHyps1])
        (LHyps = [(linc_seqSig LTyHyp)|LHyps1]),
   rule ((linc_seqBody LHypsa) ==>
          [linc_symHyp, (linc_tmList LTmHyp), linc_dot, linc_seqBody LHypsa1])
        (LHypsa = [(linc_seqHyp LTmHyp)|LHypsa1]),
   rule ((linc_seqBody LHypsb) ==>
          [linc_typeDcl LTyHypsb]) (LHypsb = [linc_seqSig LTyHypsb]),
   rule ((linc_seqBody LHypsc) ==>
          [linc_symHyp, linc_tmList LTmHypsc, linc_dot]) (LHypsc = [linc_seqHyp LTmHypsc]),

   rule ((linc_tmList TmList) ==>
          [linc_term HypTm, linc_comma, linc_tmList TmList1])
        (TmList = [HypTm|TmList1]),
   rule ((linc_tmList TmLista) ==> [linc_term HypTm2]) (TmLista = [HypTm2]),
  
     
  % definitions
  rule ((linc_defn LDefa) ==> [linc_symDefName, id DefName, linc_dot, linc_defnb LDefa1])
       (LDefa = linc_definition DefName LDefa1),
  rule ((linc_defn LDefaa) ==> [linc_symDefName, id DefName1, linc_dot])
       (LDefaa = linc_definition DefName1 nil),
  rule ((linc_defnb LDefb) ==> [linc_kindDcl LDefbKn, linc_defnb LDefb1])
       (LDefb = [(linc_kindDec LDefbKn) | LDefb1]),
  rule ((linc_defnb LDefc) ==> [linc_typeDcl LDefcTy, linc_defnb LDefc1])
       (LDefc = [(linc_typeDec LDefcTy)|LDefc1]),
  rule ((linc_defnb LDefd) ==> [linc_defClause LDefdCl, linc_defnb LDefd1])
       (LDefd = [(linc_defDec LDefdCl)|LDefd1]),
  rule ((linc_defnb LDefe) ==> [linc_kindDcl LDefeKn]) 
       (LDefe = [(linc_kindDec LDefeKn)]),
  rule ((linc_defnb LDeff) ==> [linc_typeDcl LDeffTy])
       (LDeff = [linc_typeDec LDeffTy]),
  rule ((linc_defnb LDefg) ==> [linc_defClause LDefgCl])
       (LDefg = [linc_defDec LDefgCl]),
                          
  % kind declaration
  rule ((linc_kindDcls LKnDcls) ==> [linc_kindDcl LKn1, linc_kindDcls LKn2])
       (append LKn1 LKn2 LKnDcls),
  rule ((linc_kindDcls LKnDcls') ==> [linc_kindDcl LKn3])
       (LKnDcls' = LKn3),
       
  rule ((linc_kindDcl KnDcl) ==> [linc_symKind, id_list KIDs, linc_symType, linc_dot])
       (linc_makeKindList KIDs KnDcl),
       
  % type declaration
  rule ((linc_typeDcls LTyDcls) ==> [linc_typeDcl LTy1, linc_typeDcls LTy2])
       (append LTy1 LTy2 LTyDcls),
  rule ((linc_typeDcls LTyDlcs') ==> [linc_typeDcl LTy3]) (LTyDlcs' = LTy3),

  rule ((linc_typeDcl TyDcl) ==> [linc_symType, id_list IDs, linc_type IDsTy, linc_dot])
       (linc_makeTypeList IDs IDsTy TyDcl),
                
  % definition clauses
  rule ((linc_defClause Def) ==>
            [linc_symInd, (linc_defHead Head), linc_symDef, (linc_defBody Body), linc_dot])
       (Def = defcl indcl Head Body),
  rule ((linc_defClause DefCoInd) ==>
            [linc_symCoInd, (linc_defHead Head), linc_symDef, (linc_defBody Body), linc_dot])
       (DefCoInd = defcl coindcl Head Body),       
  rule ((linc_defHead DefHead) ==> [id DHead]) (DefHead = (fvar DHead)),
  rule ((linc_defBody DefBody) ==> [linc_term BTerm]) (DefBody = BTerm),

  % terms  
  rule ((linc_term R) ==> [id R2]) (R = (fvar R2)),
  rule ((linc_term R3) ==> [lparen,(linc_term R1),rparen]) (R3 = R1),
  rule ((linc_term ER) ==> [linc_term ER1, linc_symEq, linc_term ER2]) 
       (ER = (leq @ ER1 @ ER2)),
  rule ((linc_term R6) ==> [(linc_term A6),(linc_term B6)]) (R6 = (A6 @ B6)),
  rule ((linc_term C1) ==> [id V, linc_colon, linc_type Ty, linc_symAbs, (linc_term B)]) 
        (linc_makeAbs V B H, C1 = (lam Ty H)),
  rule ((linc_term True) ==> [linc_symTrue]) (True = ltrue),
  rule ((linc_term False) ==> [linc_symFalse]) (False = lfalse),
  rule ((linc_term And) ==> 
           [(linc_term And1), linc_symAnd, (linc_term And2)])
       (And = (land @ And1 @ And2)),
  rule ((linc_term Or) ==>
           [(linc_term Or1), linc_symOr, (linc_term Or2)])
       (Or = (lor @ Or1 @ Or2)),
  rule ((linc_term Imp) ==>
           [linc_term Imp1, linc_symImp, linc_term Imp2])
       (Imp = (limp @ Imp1 @ Imp2)),
  rule ((linc_term All) ==>
           [linc_symAll, linc_term All'])
       (All = (lall @ All')),
  rule ((linc_term Exists) ==>
           [linc_symExists, linc_term Exists'])
       (Exists = (lexists @ Exists')),
       

  % types       
  rule ((linc_type TyId) ==> [id TyId']) (TyId = bt TyId'),
  rule ((linc_type TyId2) ==> [linc_symFormt]) (TyId2 = fo),       
  rule ((linc_type TyId2a) ==> [telparen, (linc_type TyId2b), rparen]) (TyId2a = TyId2b),
  rule ((linc_type TyArr) ==> 
           [(linc_type TyArr1), linc_symArr, (linc_type TyArr2)])
       (TyArr = (TyArr1 --> TyArr2)),
       
  rule ((id_list IDList) ==> [id ID1]) (IDList = [ID1]),
%  rule ((id_list IDList1) ==> [id ID2, linc_comma, id_list IDList2])
%       (IDList1 = [ID2|IDList2]),
  rule ((id_list IDList1) ==> [nontermst IDList2, id ID2])
        (IDList1 = [ID2|IDList2]),
  rule ((nontermst Nonterm) ==> [id_list Idlnt,linc_comma]) (Nonterm = Idlnt),      
  rule ((telparen) ==> [lparen]) (true)       
].

% precedence and associatvity.  needed to resovle shift-reduce conflicts:


implicitop (linc_term X) (linc_term Y) "left" 20. 
binaryop linc_symEq (linc_term X) (linc_term Y) "left" 30.
binaryop linc_symOr  (linc_term A) (linc_term B) "left" 100.
binaryop linc_symAnd (linc_term A) (linc_term B) "left" 120.
binaryop linc_symImp (linc_term A) (linc_term B) "right" 130.
unaryop linc_symAbs (linc_term A) 180.
unaryop linc_symAll (linc_term B) 180.
unaryop linc_symExists (linc_term B) 180.

binaryop linc_symArr (linc_type A) (linc_type B) "right" 20.

%% ------------


linc_makeAbs V B L :- 
   pi x\ ((linc_copy (fvar V) x :- !) => linc_copy B (L x)).



linc_makeTypeList nil Ty nil.
linc_makeTypeList [ID|L] Ty [H|LT] :-
   H = (sg (fvar ID) Ty),
   linc_makeTypeList L Ty LT.

linc_makeKindList nil nil.
linc_makeKindList [ID|L1] [(bt ID)|L2] :-
   linc_makeKindList L1 L2.





       
%% -----------