% Module printer
%
% Copyright (C) 2003 Alwen Tiu
%
%   This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or (at
% your option) any later version.
% 
%   This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
%   You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software Foundation,
% Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


% Module: printer
% Author: Alwen Tiu
% Date: 8/26/03
% Description: various predicates for formatting output of expressions, sequents, etc.

module printer.

accumulate control, list.


print_ltype Out fo :- !, output Out "o".
print_ltype Out (bt S) :- output Out S.
print_ltype Out ((A --> B) --> C) :-
   !, output Out "(", print_ltype Out A, output Out " -> ", print_ltype Out B,
   output Out ")", output Out " -> ", print_ltype Out C.
print_ltype Out (A --> B) :-
   print_ltype Out A, output Out " -> ", print_ltype Out B.      

print_lterm Out I (fvar S) :- !, output Out S.
print_lterm Out I (ltrue) :- !, output Out "true".
print_lterm Out I (lfalse) :- !, output Out "false".   
print_lterm Out I (land @ M @ N) :-
   !, output Out "(", print_lterm Out I M, output Out " & ", print_lterm Out I N, output Out ")".
print_lterm Out I (lor @ M @ N) :-
   !, output Out "(", print_lterm Out I M, output Out " ; ", print_lterm Out I N, output Out ")".
print_lterm Out I (limp @ M @ N) :-
   !, output Out "(", print_lterm Out I M, output Out " => ", print_lterm Out I N, output Out ")".
print_lterm Out I (lall @ L) :-       
   !, output Out "pi ", print_lterm Out I L.
print_lterm Out I (lexists @ L) :-
   !, output Out "sigma ", print_lterm Out I L.
print_lterm Out I (leq @ P @ Q) :-
   !, print_lterm Out I P, output Out " = ", print_lterm Out I Q.      
%print_lterm Out I (M @ (N @ P)) :-
%   !, output Out "(", print_lterm Out I M, output Out " (",
%   print_lterm Out I N, output Out " ", print_lterm Out I P, output Out ")",
%   output Out ")".
%print_lterm Out I (M @ N @ P) :-
%   !, output Out "(", print_lterm Out I M, output Out " ", print_lterm Out I N,
%   output Out " ", print_lterm Out I P, output Out ")". 
print_lterm Out I (M @ N) :-
   !, output Out "(", print_lterm Out I M, output Out " ", print_lterm Out I N,
   output Out ")".   
print_lterm Out I (lam Ty L) :-
   !, I' is I + 1, term_to_string I SI, 
   Str is "x_" ^ SI, 
   output Out "(", output Out Str, output Out ":",
   print_ltype Out Ty, output Out "\\",
   (pi x\ (pi J\print_lterm Out J x :- output Out Str) 
           => print_lterm Out I' (L x)),
   output Out ")".
   

print_signat Out nil.
print_signat Out [(sg Tm Ty) | L] :-
   output Out "type ",
   print_lterm Out 0 Tm, output Out "  ",
   print_ltype Out Ty, output Out ".\n",
   print_signat Out L.

type print_listTerms out_stream -> list tm -> o.

print_listTerms Out nil.
print_listTerms Out [T | L] :-
   output Out "hyp \n",
   pprint_lterm "    " Out 0 T,
   output Out ".\n",      
   print_listTerms Out L.


      
print_sequent Out (seq Nm LTy LHyp G) :-
   output Out "seq ", output Out Nm, output Out ".\n",
   print_signat Out LTy, 
   print_listTerms Out LHyp,
   output Out "goal \n",
   pprint_lterm "    " Out 1 G,
   output Out ".\n".

print_NumberedSequent N Out nil.
print_NumberedSequent N Out [S|L] :-
   output Out "\n%% Sequent ", term_to_string N StrN,
   output Out StrN, output Out "\n",
   print_sequent Out S,
   N' is N + 1,
   print_NumberedSequent N' Out L.
   
print_ListSequent Out nil.
print_ListSequent Out [S | L]  :-
   print_sequent Out S,
   output Out "\n",
   print_ListSequent Out L.
   
   
type print_kinds out_stream -> list ty -> o.

print_kinds Out nil.
print_kinds Out [K|L] :-
   output Out "kind ", print_ltype Out K,
   output Out " type.\n", print_kinds Out L.

type print_clauses out_stream -> list defclause -> o.

print_clauses Out nil.
print_clauses Out [(defcl indcl A B) | L] :-
   output Out "ind ", print_lterm Out 1 A, 
   output Out " <=> ", print_lterm Out 1  B,
   output Out ".\n",
   print_clauses Out L.
   
print_def Out (def Nm Kinds Sig Cls) :-
   output Out "def ", output Out Nm, output Out ".\n",
   print_kinds Out Kinds,
   print_signat Out Sig,
   print_clauses Out Cls,
   output Out "\n".
      
% ---

type pprint_lterm string -> out_stream -> int -> tm -> o.

pprint_lterm Indent Out I (ltrue) :- !, output Out Indent, output Out "true".
pprint_lterm Indent Out I (lfalse) :- !, output Out Indent, output Out "false".   
pprint_lterm Indent Out I (leq @ P @ Q) :-
   !, output Out Indent, 
   eprint_lterm Out I P, output Out " = ", eprint_lterm Out I Q.  
   
pprint_lterm Indent Out N (land @ R @ T) :-
   !, 
   I1 is Indent ^ "   ",
   output Out Indent, output Out "(\n",
   pprint_lterm I1 Out N R, 
   output Out "\n", output Out Indent,
   output Out "& \n", 
   pprint_lterm I1 Out N T,
   output Out "\n", output Out Indent, output Out ")".
   
pprint_lterm Indent Out N (lor @ R @ T) :-
   !, 
   I1 is Indent ^ "   ",
   output Out Indent, output Out "(\n",   
   pprint_lterm I1 Out N R, 
   output Out "\n", output Out Indent,
   output Out "; \n", 
   pprint_lterm I1 Out N T,
   output Out "\n", output Out Indent, output Out ")".

pprint_lterm Indent Out N (limp @ R @ T) :-
   !, 
   I1 is Indent ^ "   ",
   output Out Indent, output Out "(\n",   
   pprint_lterm I1 Out N R, 
   output Out "\n", output Out Indent,
   output Out "=> \n", 
   pprint_lterm I1 Out N T,
   output Out "\n", output Out Indent, output Out ")".

pprint_lterm Indent Out N (lall @ L) :-
   !, 
   output Out Indent, output Out "(\n",   
   output Out Indent, output Out "pi ",
   pprint_lterm Indent Out N L,
   output Out "\n", output Out Indent, output Out ")".

pprint_lterm Indent Out N (lexists @ L) :-
   !, 
   output Out Indent, output Out "(\n",   
   output Out Indent, output Out "sigma ",
   pprint_lterm Indent Out N L,
   output Out "\n", output Out Indent, output Out ")".
      
pprint_lterm Indent Out I (lam Ty L) :-
   !, I' is I + 1, term_to_string I SI, 
   Str is "x_" ^ SI, 
   output Out Str, output Out ":",
   print_ltype Out Ty, output Out "\\ \n",
   Ind1 is Indent ^ "  ",
   (pi x\ (pi J\eprint_lterm Out J x :- output Out Str) 
           => pprint_lterm Ind1 Out I' (L x)).

pprint_lterm Indent Out N L :-
   output Out Indent, 
   %print_lterm Out N L.   
   eprint_lterm Out N L.

%% ---


print_ListTermsDlmt Out nil :- output Out "".
print_ListTermsDlmt Out [T|L] :-
	pprint_lterm "" Out 0 T,
	if (L = nil) (true)
	   (output Out "\n%%\n",
	    print_ListTermsDlmt Out L).
	
print_ListSequentDlmt Out nil :-
   output Out "".
print_ListSequentDlmt Out [S|L] :-
   print_sequent Out S,
   if (L = nil) (true)
      (output Out "\n%%\n",   
       print_ListSequentDlmt Out L).
                
       
% ---- extended print term

type get_term_args tm -> list tm -> o.

get_term_args (M @ N) L :-
   get_term_args M L1,
   append L1 [N] L.

get_term_args X [X] :-
   not (X = (M @ N)).

type eprint_app_term out_stream -> int -> list tm -> o.

eprint_app_term Out I nil.
eprint_app_term Out I [T|L] :-
   if (L = nil) 
      ( eprint_lterm Out I T )
      ( 
         if (T = (lam Ty B))
            ( 
               output Out "(",
               eprint_lterm Out I T,
               output Out ") ",
               eprint_app_term Out I L
            )
            ( 
               eprint_lterm Out I T,
               output Out " ",
               eprint_app_term Out I L
            )
      ).
      
type eprint_lterm out_stream -> int -> tm -> o.

eprint_lterm Out I (fvar S) :- !, output Out S.
   
eprint_lterm Out I (M @ N) :-
   !, 
   get_term_args (M @ N) L,
   output Out "(", 
   eprint_app_term Out I L,
   output Out ")".
   
eprint_lterm Out I (lam Ty L) :-
   I' is I + 1, term_to_string I SI, 
   Str is "x_" ^ SI, 
   output Out Str, output Out ":",
   print_ltype Out Ty, output Out "\\",
   (pi x\ (pi J\eprint_lterm Out J x :- output Out Str) 
           => eprint_lterm Out I' (L x)).
